<?php

namespace WCAldi;

use WCAldi\Importer\ProductImporter;
use WCAldi\Managers\ProductsMegaMenuManager;
use WCAldi\Managers\ProductsExpresMenuManager;
use WCAldi\Managers\ProductsWeeklyMenuManager;
use WCAldi\Managers\ProductsWeekendMenuManager;
use WCAldi\Managers\ProductFieldsManager;
use WCAldi\MassiveImporter\MassiveProductsImporter;
use WCAldi\Managers\ProductStoreManager;
use WCAldi\Managers\ProductPromoManager;
use WCAldi\Managers\StoreSelectorManager;
use WCAldi\Managers\ArchiveFilterManager;
use WCAldi\Backend\ProductStoreBackend;
use WCAldi\Entities\PromosTaxonomy;
use WCAldi\Entities\SealsTaxonomy;
use WCAldi\OptionPages\IMAImportOptionPage;
use WCAldi\OptionPages\ProductsCacheOptionPage;
use WCAldi\OptionPages\ProductsJsonGeneratorOptionPage;
use WCAldi\OptionPages\ProductsMenuGeneratorOptionPage;
use WCAldi\OptionPages\StoreSelectorGeneratorOptionPage;
use WCAldi\Printers\CategoryPrinters;
use WCAldi\PrettyUrl\PrettyFilters;
use WCAldi\PrettyUrl\PrettyJsons;
use WCAldi\PrettyUrl\PrettyExpressWeek;
use WCAldi\Managers\BreadcrumbManager;
use WCAldi\Exporter\ProductsExporter;

class App {

	private $cache_database;

	public function init() {

		global $wpdb;

		$this->cache_database = $wpdb->prefix . WC_ALDI_PRODUCTS_CACHE;

		$this->defineConstants();

		// Adding modal for selecting store - It must be the first call in order to have the store always selected
		$storeSelector = new StoreSelectorManager();
		$storeSelector->init();

		$api = new Api();
		$api->registerRoutes();

		// Starting the importer
		$productImporter = new ProductImporter();
		$productImporter->init();

		// Creating the promos taxonomy
		$promosTaxonomy = new PromosTaxonomy();
		$promosTaxonomy->init();

		// Creating the seals taxonomy
		$sealsTaxonomy = new SealsTaxonomy();
		$sealsTaxonomy->init();

		// IMA Import options page
		$imaImportOptionPage = new IMAImportOptionPage();
		$imaImportOptionPage->init();

		// JSON generator Options page
		$productsJsonGeneratorOptionPage = new ProductsJsonGeneratorOptionPage();
		$productsJsonGeneratorOptionPage->init();

		// Product's menu generator Options page
		$productsMenuGeneratorOptionPage = new ProductsMenuGeneratorOptionPage();
		$productsMenuGeneratorOptionPage->init();

		// Store's list generator for the store's selector
		$storeSelectorGeneratorOptionPage = new storeSelectorGeneratorOptionPage();
		$storeSelectorGeneratorOptionPage->init();

		// Product's cache option page
		$productsCacheOptionPage = new ProductsCacheOptionPage();
		$productsCacheOptionPage->init();

		$massiveImporter = new MassiveProductsImporter();
		$massiveImporter->init();

		// Archive Filter
		$archiveFilter = new ArchiveFilterManager();
		$archiveFilter->init();

		// Products Mega Menu
		$productsMegaMenuManager = new ProductsMegaMenuManager();
		$productsMegaMenuManager->init();

		// Expres Menu
		$productsExpresMenuManager = new ProductsExpresMenuManager();
		$productsExpresMenuManager->init();

		// Weekly Menu
		$productsWeeklyMenuManager = new ProductsWeeklyMenuManager();
		$productsWeeklyMenuManager->init();

		// Weekend Menu
		$productsWeekendMenuManager = new ProductsWeekendMenuManager();
		$productsWeekendMenuManager->init();

		// Printers
		$categoryPrinters = new CategoryPrinters();
		$categoryPrinters->init();

		// Pretty Subcategories
		$prettyFilters = new PrettyFilters();
		$prettyFilters->init();

		// Pretty Json
		$prettyJsons = new PrettyJsons();
		$prettyJsons->init();

		// Pretty Express Week
		$prettyExpressWeek = new PrettyExpressWeek();
		$prettyExpressWeek->init();

		// Breadcrumb changes
		$breadcrumbManager = new BreadcrumbManager();
		$breadcrumbManager->init();

		// Adding a tab for setting the stores on the backend of the product
		$productStoreManager = new ProductStoreManager();
		$productStoreManager->init();

		// Adding a tab for setting the promos on the backend of the product (for some reason it doesn't save if it's not the first class to be called)
		$productPromoManager = new ProductPromoManager();
		$productPromoManager->init();

		// Adding new fields on the backend of the product
		$productFieldsManager = new ProductFieldsManager();
		$productFieldsManager->init();

		// Product Exporter
		$productsExporter = new ProductsExporter();
		$productsExporter->init();

		// Calling styles and scripts into the frontend (General files for all the purposes)
		add_action( 'init', [ $this, 'frontendStyle'] );

		// Calling styles and scripts into the backend
		add_action( 'admin_enqueue_scripts', [ $this, 'backendStyle'] );

		// Remove Breadcrumb
		$this->removeBreadcrumb();

		// run the install scripts upon plugin activation
		register_activation_hook( __FILE__ ,[ $this, 'wcaldi_on_install'] );
	}

	private function defineConstants() {

		global $table_prefix;

		DEFINE( 'WC_ALDI_PRODUCT_STORE_TABLE', $table_prefix . 'wcaldi_product_store' ); // Table name
		DEFINE( 'WC_ALDI_PRODUCTS_CACHE', $table_prefix . 'wcaldi_products_cache' ); // Table name
		DEFINE( 'WC_ALDI_TAX_PROMOS', 'product_promos' );
		DEFINE( 'WC_ALDI_TAX_SEALS', 'product_seals' );
		DEFINE( 'WC_ALDI_DEFAULT_COLOR', '#232e7d' );

	}

	/**
     * Adding style and scripts to the frontend
     */
    public function frontendStyle() {

		// Scripts
		wp_enqueue_script( 'wcaldi-frontend-scripts', WC_ALDI_PLUGIN_URL . 'assets/js/frontend.js', ['jquery'], '' );
		wp_localize_script( 'wcaldi-frontend-scripts', 'WC_Aldi', array(
			// URL to wp-admin/admin-ajax.php to process the request
		
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
		
			// generate a nonce with a unique ID "myajax-post-comment-nonce"
			// so that you can check it later when an AJAX request is sent
			'security' => wp_create_nonce( 'wc_aldi-ajax' )
	
		));

		// Style
		wp_enqueue_style( 'wcaldi-frontend-styles', WC_ALDI_PLUGIN_URL . "assets/css/frontend.css" );

	}
	
	/**
     * Adding style and scripts to the backend
     */
	 public function backendStyle() {
		
		// Style
		wp_enqueue_style( 'wcaldi-backend-styles', WC_ALDI_PLUGIN_URL . "assets/css/backend.css" );

	}

	protected function removeBreadcrumb() {
		add_action( 'plugins_loaded', function() {
			remove_action( 'woocommerce_before_main_content','woocommerce_breadcrumb', 20, 0);
		} );

	}

	function wcaldi_on_install() {

		global $wpdb;

		// create the ECPT metabox database table
		if($wpdb->get_var("SHOW TABLES LIKE ' . $cache_database .'") != $cache_database) 
		{
			$sql = 'CREATE TABLE " . $cache_database . " (
				`product_id` bigint(20) NOT NULL,
				`name` text,
				`description` text,
				`date` datetime DEFAULT NULL,
				`variety` varchar(50) DEFAULT NULL,
				`origin` varchar(50) DEFAULT NULL,
				`brand` varchar(50) DEFAULT NULL,
				`unit_measure` varchar(20) DEFAULT NULL,
				`unit_content` varchar(20) DEFAULT NULL,
				`legal_text` varchar(200) DEFAULT NULL,
				`nutritional_category` varchar(50) DEFAULT NULL,
				`pack` varchar(30) DEFAULT NULL,
				`outstanding_sentence` varchar(150) DEFAULT NULL,
				`sku` varchar(10) DEFAULT NULL,
				`product_family` varchar(200) DEFAULT NULL,
				`featured_img` varchar(200) DEFAULT NULL,
				`images` text,
				`permalink` varchar(200) DEFAULT NULL,
				`order_product` int(11) DEFAULT NULL,
				`meta_keywords` varchar(100) DEFAULT NULL,
				`meta_description` varchar(160) DEFAULT NULL,
				`product_type` varchar(12) DEFAULT NULL,
				`aldi_family` varchar(4) DEFAULT NULL,
				`aldi_code` varchar(10) DEFAULT NULL,
				`seals` text,
				`promo_start_date` datetime DEFAULT NULL,
				`promo_end_date` datetime DEFAULT NULL,
				`promo_normal_price` decimal(2,2) DEFAULT NULL,
				`promo_offer_price` decimal(2,2) DEFAULT NULL,
				`promo_normal_unit_price` varchar(45) DEFAULT NULL,
				`promo_offer_unit_price` varchar(45) DEFAULT NULL,
				`visibility` varchar(15) DEFAULT NULL
			);';

			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}

	}

}
