<?php

namespace WCAldi\Importer;

use WCAldi\Repositories\StoreRepo;
use WCAldi\Repositories\ProductStoreRepo;
use WCAldi\Repositories\PromosRepo;
use WCAldi\Repositories\TaxonomyRepo;
use WCAldi\Repositories\ProductCategoriesRepo;

class ProductImporter {

	public function init() {
		add_action( "woocommerce_rest_insert_product", [ $this, 'updateProductExtensions' ], 10, 3 );
	}

	/**
	 * @param \WP_Post $product
	 * @param \WP_REST_Request $request
	 * @param boolean $creating
	 */
	public function updateProductExtensions( $product, $request, $creating ) {

		$data = json_decode( $request->get_body() );

		if( isset($data->product_store) ) {
			$this->updateProductStore( $product, $data->product_store );
		}
		
		if( isset($data->promos) ) {
			$this->updateProductPromos( $product, (array) $data->promos );
		} else {
			$this->updateProductPromos( $product, [] );
		}

		if( isset($data->seals) ) {
			$this->updateProductSeals( $product, $data->seals );
		} else {
			$this->updateProductSeals( $product, [] );
		}

		if( isset($data->aldi_categories) ) {
			$this->updateProductCategories( $product, $data->aldi_categories );

			$saveCompleteCategoryTree = new ProductCategoriesRepo();
			$saveCompleteCategoryTree->saveCompleteCategoryTree();
		} else {
			$this->updateProductCategories( $product, [] );
		}

		if( isset($data->metatags) ) {
			$this->updateMetaTags( $product, $data->metatags );
		}

		if( isset($data->aldi_code) ) {
			$this->updateAldiCode( $product, $data->aldi_code );
		}

		if( isset($data->aldi_family) ) {
			$this->updateAldiFamily( $product, $data->aldi_family );
		}

		$this->setVisibilityToVisible( $product );

	}

	public function updateProductStore( $product, $data ) {

		$storeManager = new StoreRepo();
		$storeCodes   = $storeManager->getCodeIdRelation();

		$rows = [];
		foreach ( $data as $item ) {

			// Checking that the store exists in the system
			if( $storeCodes[ $item->code_store ] != 0 ) {
				$rows[] = [
					'product_id'   => $product->ID,
					'store_id'     => $storeCodes[ $item->code_store ],
					'is_available' => $item->status == 'OS',
					'price'        => (float) $item->price,
					'price_unit'   => str_replace('.',',',$item->price_unit),
				];
			}

		}

		$productStoreRepo = new ProductStoreRepo();
		$productStoreRepo->deleteByProductId( $product->ID );
		$productStoreRepo->insertData( $rows );
		
	}

	public function updateProductPromos( $product, $data ) {

		$promos = new PromosRepo();
		$promos->setProductPromos($product->ID, $data );

	}

	public function updateProductSeals( $product, $data ) {

		$seals = new TaxonomyRepo();
		$seals->setProductTaxonomies($product->ID, $data, WC_ALDI_TAX_SEALS );

	}

	public function updateProductCategories( $product, $data ) {

		$categories = new TaxonomyRepo();
		$categories->setProductTaxonomies($product->ID, $data, 'product_cat');

	}

	public function updateMetaTags( $product, $data ) {

		update_post_meta( $product->ID, '_yoast_wpseo_title', $data->title );
		update_post_meta( $product->ID, '_yoast_wpseo_metadesc', $data->description );
		update_post_meta( $product->ID, '_yoast_wpseo_focuskw', $data->keywords );
		update_post_meta( $product->ID, '_yoast_wpseo_focuskw_text_input', $data->keywords );

	}

	public function updateAldiCode( $product, $data ) {

		update_post_meta( $product->ID, '_raw_code', $data );

	}

	public function updateAldiFamily( $product, $data ) {
		
		update_post_meta( $product->ID, '_aldi_family', $data );

	}

	/**
	 * It sets a product's visibility to visible
	 */
	private function setVisibilityToVisible( $product ) {

		update_post_meta( $product->ID, '_visibility', 'visible' );

	}

	/**
	 * IMA and Wordpress are not sharing the same promos structure. This function transform the IMA promos into Wordpress ones
	 * 
	 * @param string $promo with the promos separated by |
	 * 
	 * @return string with the promos transformed for Wordpress, separated again by |
	 */
	public function transformPromoToWordpress( $promo ) {

		$arrayPromos = explode('|', $promo );

		// Transforming only for those with only two levels
		if( ( $arrayPromos[0] == 'Especiales de la semana' )
			|| ( $arrayPromos[0] == 'Ofertas de la semana' ) ) {
				if( $arrayPromos[1] == 'Gran temática' ) {
					return "Promociones de la semana|".$arrayPromos[1];
				} else {
					return "Promociones de la semana|".$arrayPromos[0]."|".$arrayPromos[1];
				}
		} elseif ( $arrayPromos[0] == 'Ideas exprés' ) {
			switch ($arrayPromos[1]) {
				case 'Lunes': return $arrayPromos[0].'|Ideas exprés lunes|'.$arrayPromos[2];
					break;
				case 'Miércoles': return $arrayPromos[0].'|Ideas exprés miércoles|'.$arrayPromos[2];
					break;
				case 'Entre plantas': return $arrayPromos[0].'|'.$arrayPromos[1].'|'.$arrayPromos[2];
					break;
				case 'Sábado': return $arrayPromos[0].'|Ideas exprés sábado|'.$arrayPromos[2];
					break;
			}
		} elseif ( $arrayPromos[0] == 'Destacados fin de semana' ) {
			if( $arrayPromos[1] == "Frescos" ) {
				return "Destacados del fin de semana|Destacados fin de semana frescos";
			} else {
				return "Destacados del fin de semana|Destacados fin de semana|".$arrayPromos[1];
			}
		} else {
			return '';
		}

	}
	
}