<?php

namespace WCAldi\MassiveImporter;

use WCAldi\Importer\ProductImporter;
use WCAldi\Repositories\ProductRepo;
use WCAldi\Repositories\ProductCategoriesRepo;
use WCAldi\Cache\ProductsCache;

class MassiveProductsImporter {

    private $products;
    private $start_date;
    private $periodicity;
    private $limitProducts = 5;

	public function init() {

        // Execute CRON
        add_action('ima_import_cron', array( $this, 'importIMAProductsLastDay' ) );

        // Uncomment for calling the importer in any URL without AJAX, so it's easier to debug
        // add_action('init', [ $this, 'importIMAProductsLastDay'] );

    }

    /**
     * It's an alias for the readSaveJson function. It gets all the products.
     */
    public function importIMAProducts() {

        $this->readSaveJson();

    }

    /**
     * It's an alias for the readSaveJson function. It only reads the last 24 hours
     */
    public function importIMAProductsLastDay() {

        $this->readSaveJson( 27 );

    }

    /**
     * It's an alias for the readSaveJson function
     */
     public function importIMAProductsLastHours( $hours ) {

        $this->readSaveJson( $hours );

    }

    /**
     * It reads from the JSON and save the products in Woocommerce
     *
     * @param int|null $lastHours
     */
     public function readSaveJson( $lastHours = null ) {
		ini_set('max_execution_time', 24 * 3600); // 1 day

        $this->setImportStatus( 1 );

        $date = date("Y-m-d");
		$this->saveHeaderLog(WC_ALDI_LOG_FILE . '_' . "$date.txt");

        $offset = 0;

        while( ( $jsonProducts = $this->getIMAProducts( $offset, $this->limitProducts, $lastHours ) ) && $this->getImportStatus() ) {

            $this->saveProductsInWoocommerce( $jsonProducts );
            $offset += $this->limitProducts;

        }

		$saveCompleteCategoryTree = new ProductCategoriesRepo();
        $saveCompleteCategoryTree->saveCompleteCategoryTree();

        $productsCache = new ProductsCache();
        $productsCache->saveCache();

        $this->setImportStatus( 0 );

    }

    /**
     * It sets the status of the import.
     * 
     * @param boolean $status - True for importing, false for not
     */
    public function setImportStatus( $status ) {

        \update_option('ima_import_status', $status, 'no');

    }

    /**
     * It gets the status of the import.
     */
    public function getImportStatus() {

        if( \get_option('ima_import_status') ) {
            $status = \get_option('ima_import_status');
        } else {
            $status = false;
        }

        return $status;

    }

    /**
     * It reads all the products from IMA
     *
     * @param int|null $offset - The position where to start reading
     * @param int|null $limit - How many products to retrieve in a single json
     * @param int $lastHours (optional) - Get only the products updated in the x last hours
     *
     * @return array of info of the products
     */
    public function getIMAProducts( $offset = null, $limit = null, $lastHours = null ) {

        // Reading the products into small parts to prevent php memory problems
        $url = $this->createJsonUrl( $offset, $limit, $lastHours );
        $result = file_get_contents( $url );


		// PARCHE: guardar registro de JSON importados
		$fichero = WC_ALDI_PLUGIN_PATH . 'logs/log_json_' . date('Y-m-d') . '.log';
		$cabecera = "
			== JSON " . date('Y-m-d H:i:s') . " ==
			offset: $offset
			limit: $limit
			$lastHours : $lastHours
			==========
		";

		$fp = fopen($fichero, 'a');
		fwrite($fp, $cabecera);
		fwrite($fp, $result);
		fclose($fp);


        // Will dump a beauty json :3
        if( $result != '[]' ) {
            return json_decode($result, true);
        }

        return null;

    }

    /**
     * It creates the url of the JSON with the products. It gets the base URL and the api key from the Wordpress options
     *
     * @param int|null $offset - The position where to start reading
     * @param int|null $limit - How many products to retrieve in a single json
     * @param int|null $lastHours - Select only products updated in the last x hours
     *
     * @return string - String with the URL
     */
    public function createJsonUrl( $offset = null, $limit = null, $lastHours = null ) {

        // Retrieving the IMA options
        $ima_import_options = get_option( 'ima_import_options' );
        $this->ApiUrl = $ima_import_options['ima_import_ima_url'];
        $this->ApiKey = $ima_import_options['ima_import_ima_apikey'];
        $onlyImagesNotUploaded = "&set_images_unchanged=0";
        if( $ima_import_options['ima_import_ima_images'] ) {
            $onlyImagesNotUploaded = "&set_images_unchanged=1";
        }
        
        // Setting the URL to retrieve the content
        $url = $this->ApiUrl . "?apikey=" . $this->ApiKey . $onlyImagesNotUploaded;

        if( $lastHours != null ) {
            $url .= '&lasthours=' . $lastHours;
        }

        if( $offset != null ) {
            $url .= '&offset=' . $offset;
        }

        if( $limit != null ) {
            $url .= '&limit=' . $limit;
        }

        return $url;

    }

    /**
     * It saves the products into Woocommerce
     *
     * @param array $products - An array with all the products to save in Woocommerce
     *
     * @return boolean - True|false depending on the success of the operation
     */
    public function saveProductsInWoocommerce( $products ) {

        $productImporter = new ProductImporter();
        $d = date("Y-m-d");

        if( !empty( $products ) ) {

            foreach( $products as $product ) {

                $productId = wcaldi_get_product_id_by_sku( $product['sku'] );

                // Checking if we have to delete the product
                if( $product['status'] == 'published' ) {

                    try {

                        $prData = (object) array('ID' => $productId);
                        $productRepo = new ProductRepo();
                        $currentProduct = new \WC_Product( $productId );
    
                        if( $productId == null ) {
                            $prData = (object) array('ID' => $productRepo->insertProduct( $product ) );
                            $this->saveLog( $product['sku'], $prData->ID, 'created', WC_ALDI_LOG_FILE.'_'.$d.'.txt' );
                        } else {
                            $productRepo->updateProduct( $productId, $product );
                            $this->saveLog( $product['sku'], $productId, 'updated', WC_ALDI_LOG_FILE.'_'.$d.'.txt' );
                        }
    
                        if( isset( $product['product_store'] ) && !empty( $product['product_store'] ) ) {
                            $productImporter->updateProductStore( $prData, json_decode(json_encode($product['product_store']), FALSE) );
                        }
    
                        if( isset( $product['promos'] ) ) {
                            $productImporter->updateProductPromos( $prData, $product['promos'] );
                        } else {
                            $productImporter->updateProductPromos( $prData, [] );
                        }
    
                        if( isset( $product['seals'] ) ) {
                            $productImporter->updateProductSeals( $prData, $product['seals'] );
                        } else {
                            $productImporter->updateProductSeals( $prData, [] );
                        }
    
                        if( isset( $product['aldi_categories'] ) ) {
                            $productImporter->updateProductCategories( $prData, $product['aldi_categories'] );
                        } else {
                            $productImporter->updateProductCategories( $prData, [] );
                        }
    
                        if( isset( $product['metatags'] ) ) {
                            $productImporter->updateMetaTags( $product, $product['metatags'] );
                        }
    
                        if( isset( $product['images'] ) && !empty( $product['images'] ) ) {
    
                            $this->setImages( $prData->ID, $product['images'] );
    
                        }

                        if( isset( $product['aldi_code'] ) && !empty( $product['aldi_code'] ) ) {
                            
                            $productImporter->updateAldiCode( $prData, $product['aldi_code'] );
    
                        }

                        if( isset( $product['aldi_family'] ) && !empty( $product['aldi_family'] ) ) {
                            
                            $productImporter->updateAldiFamily( $prData, $product['aldi_family'] );
    
                        }
    
                        // ATTRIBUTES
                        if( isset( $product['unit_measure'] ) ) {
                            $attributes['Unidad de medida'] = $product['unit_measure'];
                        } else {
                            $attributes['Unidad de medida'] = '';
                        }
    
                        if( isset( $product['unit_content'] ) ) {
                            $attributes['Contenido unidad'] = $product['unit_content'];
                        } else {
                            $attributes['Contenido unidad'] = '';
                        }
    
                        if( isset( $product['pack'] ) ) {
                            $attributes['Pack'] = $product['pack'];
                        } else {
                            $attributes['Pack'] = '';
                        }
    
                        if( isset( $product['brand'] ) ) {
                            $attributes['Marca'] = $product['brand'];
                        } else {
                            $attributes['Marca'] = '';
                        }
    
                        if( isset( $product['legal_text'] ) ) {
                            $attributes['Texto legal'] = $product['legal_text'];
                        } else {
                            $attributes['Texto legal'] = '';
                        }
    
                        if( isset( $product['outstanding_sentence'] ) ) {
                            $attributes['Frase destacada'] = $product['outstanding_sentence'];
                        } else {
                            $attributes['Frase destacada'] = '';
                        }

                        if( isset( $product['nutritional_category'] ) && !is_null( $product['nutritional_category'] ) ) {
                            $attributes['Categoría nutricional'] = $product['nutritional_category'];
                        } else {
                            $attributes['Categoría nutricional'] = '';
                        }

                        if( isset( $product['variety'] ) && !is_null( $product['variety'] ) ) {
                            $attributes['Variedad'] = $product['variety'];
                        } else {
                            $attributes['Variedad'] = '';
                        }

                        if( isset( $product['origin'] ) && !is_null( $product['origin'] ) ) {
                            $attributes['Origen'] = $product['origin'];
                        } else {
                            $attributes['Origen'] = '';
                        }
    
                        $productRepo->wcproduct_set_attributes( $prData->ID, $attributes );

                    } catch (\Exception $e) {
                        $this->saveLog($product['sku'], '?', 'error', WC_ALDI_LOG_FILE . '_' . $d . '.txt');
                    }

                } else {

                    wp_delete_post( $productId, true ); // We delete the product

                    $this->saveLog($product['sku'], $productId, 'delete', WC_ALDI_LOG_FILE . '_' . $d . '.txt');

                }

            }

        }

    }

    /**
     * It creates a log of the products that it has been updating
     *
     * @param string $productSku - A string with the SKU of the product that is being added or updated
     * @param int $productWpId - The inner Wordpress ID for the product
     * @param string $mode - Message type ('created', 'updated', 'error')
     * @param string $fileName - The filename where to save the data
     */
     public function saveLog($productSku, $productWpId, $mode, $fileName) {
        
        $d = date("Y-m-d H:i");
        $file = WC_ALDI_PLUGIN_PATH . 'logs/' . $fileName;
        $fp = fopen($file, 'a');

        fwrite($fp,
			'Date: ' . $d . ' - Product: ' . $productSku . ' - WP_Id: ' . $productWpId . ' ' . $mode . PHP_EOL
		);

        fclose($fp);

    }

    /**
     * Create the header in the log file
     *
     * @param string $fileName - The filename where to save the data
     */
	public function saveHeaderLog($fileName) {
		$date = date('Y-m-d H:i');
        $file = WC_ALDI_PLUGIN_PATH . 'logs/' . $fileName;
        $fp   = fopen($file, 'a');

		fwrite($fp, '--------------------' . PHP_EOL);
		fwrite($fp, "[IMPORT] Date: $date" . PHP_EOL);
		fwrite($fp, '--------------------' . PHP_EOL);
	}


    /**
     * It reads and print the last updates made by the importer from the log file
     *
     * @param int $max - Maximum number of actions to show. 0 for infinite. Default 0
     */
     public function printUpdatesLog( $max = 0 ) {
        
        $i = 0;

        if( $fileName =  $this->getLastUpdatedFile( WC_ALDI_PLUGIN_PATH . 'logs/' ) ) {

            echo "<h4>".$fileName."</h4><br>";
            
            if( file_exists( WC_ALDI_PLUGIN_PATH . 'logs/' . $fileName) ) {
    
                $file = fopen( WC_ALDI_PLUGIN_PATH . 'logs/' . $fileName, "r");
    
                // Output one line until end-of-file
                while( !feof($file) && ( $i <= $max ) ) {
                    
                    echo fgets($file) . "<br>";
        
                    if ( $max != 0 ) {
                        $i++;
                    }
        
                }
        
                fclose($file);
            } else {
                echo "No se ha encontrado actividad previa.";
            }
        } else {
            echo "No se ha encontrado actividad previa.";
        }
        

    }

    /**
     * It gets the last update file inside a folder
     *
     * @param string $path - Path to the folder
     *
     * @return string - File name
     */
    function getLastUpdatedFile( $path ) {

        $dir = opendir($path);
        $list = array();
        while( $file = readdir($dir) ){
            if ($file != '.' and $file != '..'){
                // add the filename, to be sure not to
                // overwrite a array key
                $mtime = filemtime( $path . $file ) . ',' . $file;
                $list[$mtime] = $file;
            }
        }
        closedir($dir);
        krsort($list);

        if( !empty($list) ) {
            reset($list);
            return current($list);
        } else {
            false;
        }
        
    }


    /**
     * It uploads images to the Wordpress Media Gallery and associate them to a concrete product
     *
     * @param int $productId - Product Id in the wordpress database
     * @param array images - Array of images like that - [ 'src' => url, 'has_changed' = boolean ]
     */
    public function setImages( $productId, $images ) {
        
        if ( is_array( $images ) ) {
            $gallery = array();

            foreach ( $images as $image ) {

                $noChanges = 1;

                if ( $image['has_changed'] == 1 ) {

                    $attachment_id = 0;
                    $error = false;

                    if ( isset( $image['src'] ) ) {

                        $upload = wc_rest_upload_image_from_url( esc_url_raw( $image['src'] ) );

                        if ( is_wp_error( $upload ) ) {
                        } else {
                            $attachment_id = wc_rest_set_uploaded_image_as_attachment( $upload, $productId );
                            $noChanges = 0;
                        }

                    }

                    if( $noChanges == 0 ) {
                        if ( ! wp_attachment_is_image( $attachment_id ) ) {
                            throw new WC_REST_Exception( 'woocommerce_product_invalid_image_id', sprintf( __( '#%s is an invalid image ID.', 'woocommerce' ), $attachment_id ), 400 );
                        }

                        if ( isset( $image['position'] ) && 0 === absint( $image['position'] ) ) {
                            set_post_thumbnail($productId, $attachment_id );
                        } else {
                            $gallery[] = $attachment_id;
                        }

                        // Set the image alt if present.
                        if ( ! empty( $image['alt'] ) ) {
                            update_post_meta( $attachment_id, '_wp_attachment_image_alt', wc_clean( $image['alt'] ) );
                        }

                        // Set the image name if present.
                        if ( ! empty( $image['name'] ) ) {
                            wp_update_post( array( 'ID' => $attachment_id, 'post_title' => $image['name'] ) );
                        }
                    }


                }

            }

            if( $noChanges == 0 ) {
                $this->setGalleryImages( $productId, $gallery );
            }

        } else {

            $this->setGalleryImages( $productId, array() );

        }

    }

    public function setGalleryImages( $productId, $gallery ) {

        $list_id = implode(',', $gallery);
        update_post_meta($productId, '_product_image_gallery', $list_id);

    }

}
