<?php

namespace WCAldi\Repositories;

use WCAldi\Repositories\ProductStoreRepo;
use WCAldi\Repositories\ProductRepo;
use WCAldi\Repositories\PromosRepo;


class ProductPromosRepo {

	protected $taxonomy = WC_ALDI_TAX_PROMOS;
	protected $promoPropertiesMethods;
	protected $storeId;

	public function __construct() {
		if (isset($_COOKIE['current_store']['id'])) {
			$this->storeId = $_COOKIE['current_store']['id'];
		}
	}


	/**
	 * It gets the subcategories of a category, and adds the products that belong
	 * to them.
	 *
	 * @param int $promoId - Id of the promo.
	 * @param int $limit - Maximum number of elements to retrieve - null for no limit
	 *                   - Default null.
	 * @param int $offset - Where to start getting the info from - Default 0.
	 * @param string $orderBy - How to sort the products - alphabetical,
	 *                        alphabetical_inverse, price, price_inverse
	 *                        - Default alphabetical.
	 * @param boolean $ignoreWithoutPrice - Whether showing the products without price - True for ignoring, default true
	 *
	 * @return array
	 */
	public function getProductsInCategories(
		$promoId,
		$limit = null,
		$offset = 0,
		$orderBy = 'alphabetical',
		$addWeek = 0,
		$ignoreWithoutPrice = true
	) {
		$productsCat = [];

		if( $this->storeId != '' ) {

			$term = get_term_by( 'id', $promoId, 'product_promos' );

			$promoDates = $this->getDateRangeBySlug($term->slug, $addWeek);

			if ($term->slug === 'ultimos-dias') {
				$term    = get_term_by( 'slug', 'sabado-expres', 'product_promos' );
				$promoId = $term->term_id;
			}

			if (!$promoDates['life_start']) {
				return [];
			}

			$subCategories = $this->getSubCategories($promoId);

			$products = $this->getProductsInCategoriesBBDD(
				$promoId,
				$promoDates,
				$limit,
				$offset,
				$orderBy,
				$ignoreWithoutPrice
			);

			// Create and fill the first category with no title.
			$productsCat[0]['name']     = null;
			$productsCat[0]['id']       = null;
			$productsCat[0]['slug']     = null;
			$productsCat[0]['products'] = [];

			foreach ($products as $product) {
				if (!$product['is_subcategory']) {
					$productsCat[0]['products'][] = $product;
				}
			}


			// Create and fill the subcategories.
			$i = 1;

			foreach ($subCategories as $category) {
				$productsCat[$i]['name']     = $category['name'];
				$productsCat[$i]['id']       = $category['id'];
				$productsCat[$i]['slug']     = $category['slug'];
				$productsCat[$i]['products'] = [];

				foreach ($products as $product) {
					if ($product['category_id'] == $category['id']) {
						$productsCat[$i]['products'][] = $product;
					}
				}

				$i++;
			}

		}

		return $productsCat;
	}

	/**
	 * It gets the promotions data
	 * 
	 * @param array $promotions_slugs - Array with the slugs of the promotions to retrieve
	 * 
	 * @return array
	 */
	public function getSelectDateRangesBySlug( $promotion_slugs ) {

		$promoRanges = [];

		foreach( $promotion_slugs as $p ) {

			$promoRanges[$p] = $this->getDateRangeBySlug( $p );

		}

		return $promoRanges;

	}


	/**
	 * It gets the subcategories of a concrete category
	 *
	 * @param int $promoId
	 *
	 * @return array - An array with all the subcategories.
	 */
	public function getSubCategories($promoId) {
		// Get terms.
		$terms = get_terms($this->taxonomy, [
			'parent'     => $promoId,
			'hide_empty' => false
		]);

		if (!$terms) {
			return [];
		}


		// Order terms alphabetically.
		$ordered = false;

		while (!$ordered) {
			$ordered = true;
			for ($i = 0; $i < count($terms) - 1; $i++) {
				$comp = strcasecmp($terms[$i]->name, $terms[$i + 1]->name);

				if ($comp > 0) {
					$temp          = $terms[$i];
					$terms[$i]     = $terms[$i + 1];
					$terms[$i + 1] = $temp;
					$ordered      = false;
				}
			}
		}


		// Generate subcategories.
		$subCategories = [];
		$i = 0;

		foreach ($terms as $t) {
			$subCategories[$i] = [
				'id'   => $t->term_id,
				'name' => $t->name,
				'slug' => $t->slug
			];

			$i++;
		}

		return $subCategories;
	}


	/**
	 * It gets all the products belonging to a category and its subcategories.
	 *
	 * @param int $promoId - Id of the category.
	 * @param string $promoDates Range of dates to be shown: two strings in English
	 *                           format ('Y-m-d').
	 * @param int $limit - Maximum number of elements to retrieve - null for no limit
	 *                   - Default null.
	 * @param int $offset - Where to start getting the info from - Default 0
	 * @param string $orderBy - How to sort the products - alphabetical,
	 *                        alphabetical_inverse, price, price_inverse, random
	 *                        - Default alphabetical.
	 * @param array $families - Families to retrieve products from - Empty array for
	 *                        all of them - Default empty.
	 * @param boolean $ignoreWithoutPrice - Whether showing the products without price - True for ignoring, default true
	 *
	 * @return array
	 */
	public function getProductsInCategoriesBBDD(
		$promoId,
		$promoDates,
		$limit = null,
		$offset = 0,
		$orderBy = 'alphabetical',
		$ignoreWithoutPrice = true
	) {
		global $wpdb;

		if( $this->storeId == '' ) { return []; }

		// Prepare query.
		$query = $this->makeProductsInPromosQuery($promoId, $promoDates, false, $ignoreWithoutPrice);

		switch ($orderBy) {
			case 'alphabetical':
				$query .= ', p.post_title ASC';
				break;
			case 'alphabetical_inverse':
				$query .= ', p.post_title DESC';
				break;
			case 'price':
				$query .= ', wps.price ASC';
				break;
			case 'price_inverse':
				$query .= ', wps.price DESC';
				break;
			case 'random':
				$query .= ', RAND()';
				break;
		}

		if ($limit !== null) {
			$query .= " LIMIT " . $offset . "," . $limit;
		}


		// Get product results (execute query).
		$results = $wpdb->get_results($query, ARRAY_A);

		if (!$results) {
			return [];
		}


		$products = [];
		$productRepo = new ProductRepo();

		foreach ($results as $result) {
			$productId   = $result['id'];
			$contentUnit = $productRepo->getAttribute($productId, 'Pack');

			// Set thumbnail.
			if (has_post_thumbnail($productId)) {
				$thumbnailId = get_post_thumbnail_id($productId);

				$thumbnail['url'] = get_the_post_thumbnail_url($productId, 'medium');
				$thumbnail['alt'] = get_post_meta($thumbnailId, '_wp_attachment_image_alt', true);
			} else {
				$thumbnail = null;
			}

			if ($result['is_subcategory'] === '1' || !isset($products[$productId])) {
				$products[$productId] = [
					'id'             => $productId,
					'name'           => $result['name'],
					'description'    => $result['description'],
					'permalink'      => get_the_permalink( $productId ),
					'price'          => $result['price'],
					'price_unit'     => $result['price_unit'],
					'attributes'     => $result['attributes'],
					'content_unit'   => strtolower($contentUnit),
					'thumbnail'      => $thumbnail,
					'category_id'    => $result['category_id'],
					'category_name'  => $result['category_name'],
					'is_subcategory' => $result['is_subcategory'] == 1
				];
			}
		}

		return $products;
	}


	/**
	 * Building a query for getting the products in the categories query
	 *
	 * @param int $promoId.
	 * @param string $promoDates Range of dates to be shown: two strings in English
	 *                           format ('Y-m-d').
	 * @param boolean $onlyCount - Only count instead of getting the values
	 *        - Default false.
	 * @param boolean $ignoreWithoutPrice - Whether showing the products without price - True for ignoring, default true
	 *
	 * @return string - The query.
	 */
	public function makeProductsInPromosQuery(
		$promoId,
		$promoDates,
		$onlyCount = false,
		$ignoreWithoutPrice = true
	) {
		global $wpdb;

		// Get time strings between the date range.
		$iTime = $promoDates['life_start'];
		$timeArray = [];

		while ($iTime <= $promoDates['life_end']) {
			$timeArray[] = date("'Y-m-d','Y-m-d 00:00:00'", $iTime);

			$iTime += 3600 * 24;
		}

		$timeStrings = implode(',', $timeArray);


		if ($onlyCount) {
			$data = "COUNT(p.id) AS `total`";
		} else {
			$data = "
				p.id                    AS `id`,
				p.post_title            AS `name`,
				p.post_content          AS `description`,
				wps.price               AS `price`,
				wps.price_unit          AS `price_unit`,
				attributes.meta_value   AS `attributes`,
				t.term_id               AS `category_id`,
				t.name                  AS `category_name`,
				t.term_id != {$promoId} AS `is_subcategory`
			";
		}

		$getWithoutPrice = 'INNER';
		if( $ignoreWithoutPrice == false ) {
			$getWithoutPrice = 'LEFT';
		}

		$query = "
			SELECT {$data}
			  FROM {$wpdb->prefix}term_relationships AS tr
			       /* Get the post: the product. */
			       INNER JOIN {$wpdb->posts} AS p
			               ON p.id = tr.object_id
			              AND p.post_type = 'product'
			              AND p.post_status = 'publish'

			       /* Check that the product is in the current store. */
			       {$getWithoutPrice} JOIN {$wpdb->prefix}wcaldi_product_store AS wps
			               ON wps.product_id = p.id
			              AND wps.store_id = {$this->storeId}
			              AND wps.is_available IS TRUE

			       /* Check that the product is enabled. */
			       INNER JOIN {$wpdb->postmeta} AS pm
			               ON pm.post_id = p.id
			              AND pm.meta_key = '_visibility'
			              AND pm.meta_value IN ('catalog', 'visible')
			       
                   /* Check that the product is in the date range. */
			       INNER JOIN {$wpdb->postmeta} AS pmt
			               ON pmt.post_id = p.id
			              AND pmt.meta_key = 'wcaldi_promo_startdate'
			              AND pmt.meta_value IN ({$timeStrings})

                   LEFT JOIN wp_term_relationships AS ctr
                          ON ctr.object_id = tr.object_id

			       LEFT JOIN wp_term_taxonomy AS ctt
			              ON ctt.term_taxonomy_id = ctr.term_taxonomy_id
                         AND (ctt.parent = {$promoId}
						   OR ctt.term_taxonomy_id = {$promoId})
						   
				   LEFT JOIN {$wpdb->postmeta} AS attributes
				           ON attributes.post_id = p.id
						  AND attributes.meta_key = '_product_attributes'

                   INNER JOIN wp_terms AS t
                           ON  t.term_id = ctt.term_taxonomy_id

			 WHERE tr.term_taxonomy_id = {$promoId}
			 ORDER BY t.term_id != {$promoId} DESC
		";

		return $query;
	}

	/**
	 * Get the all promos for the menu.
	 *
	 * @return array with ['slug', 'title', 'icon_url', 'icon_alt', 'description'].
	 */
	public function getAllPromos() {
		$expresPromos     = $this->getSubPromos('ideas-expres');
		$weeklyPromos     = $this->getSubPromos('promociones-semana');
		$weekendPromos    = $this->getSubPromos('destacados-fin-semana');

		return array_merge($expresPromos, $weeklyPromos, $weekendPromos);
	}

	/**
	 * Get the promos for the menu.
	 *
	 * @param string $promoSlug
	 * @param int $addWeek - It allows to navigate to future or past weeks - Default 0
	 * @param boolean $returnEndDate - Whether return the end date - Default true
	 * 
	 * @return array with ['slug', 'title', 'icon_url', 'icon_alt', 'description'].
	 */
	public function getSubPromos($promoSlug, $addWeek = 0, $returnEndDate = true) {
		$promos    = [];
		$promoData = $this->getChildrenPromoData( $promoSlug );
		setlocale(LC_TIME, 'es_ES.UTF-8');

		foreach ($promoData as $promo) {
			// Check that the promo is active.
			$active = get_field('promo_is_active', $this->taxonomy . '_' . $promo['id']);
			$isActive = ($active == 1 || $active == '')? 1 : 0;

			if ($isActive) {
				$timeRange  = $this->getDateRangeBySlug($promo['slug'], $addWeek);

				if ($timeRange['life_start'] && $timeRange['life_end']) {

					$title = get_field('promo_title', $this->taxonomy . '_' . $promo['id']);
					$icon  = get_field('promo_icon', $this->taxonomy . '_' . $promo['id']);
					$color = get_field('promo_color', $this->taxonomy . '_' . $promo['id']);

					if (!$title) {
						$title = $promo['name'];
					}
					if (!$color) {
						$color = WC_ALDI_DEFAULT_COLOR;
					}

					$startDayNum   = date('d', $timeRange['life_start']);
					$startMonthNum = date('m', $timeRange['life_start']);
					$startDayName  = date_i18n('l', $timeRange['life_start']);

					$endDayNum   = date('d', $timeRange['life_end']);
					$endMonthNum = date('m', $timeRange['life_end']);
					$endDayName  = date_i18n('l', $timeRange['life_end']);

					$description = "Desde ".$this->transformWeekday($startDayName)." $startDayNum/$startMonthNum";
					if( $returnEndDate ) {
						$description .= " a ".$this->transformWeekday($endDayName)." $endDayNum/$endMonthNum";
					}

					$promos[$promo['slug']] = [
						'id'          => $promo['id'],
						'key'         => $promo['slug'],
						'url'         => get_term_link(intval($promo['id'])),
						'title'       => $title,
						'icon_url'    => $icon['url'],
						'icon_alt'    => $icon['alt'],
						'description' => $description,
						'color'       => $color
					];
				}
			}
		}

		return $promos;
	}

	/**
	 * The client wants a specific way of showing the week days
	 * 
	 * @param string $weekday
	 * 
	 * @return string
	 */
	private function transformWeekday( $weekday ) {

		switch ($weekday) {
			case 'Lunes': return 'lun.';
			case 'Martes': return 'mart.';
			case 'Miércoles': return 'miérc.';
			case 'Jueves': return 'juev.';
			case 'Viernes': return 'vier.';
			case 'Sábado': return 'sáb.';
			case 'Domingo': return 'dom.';
		}

	}

	/**
	 * Get the expres promos' data.
	 *
	 * @return array with ['id', 'slug', 'name'].
	 */
	private function getExpresPromoData() {
		global $wpdb;

		$query = "
			SELECT ct.term_id AS id,
			       ct.slug    AS slug,
			       ct.name    AS name
			  FROM {$wpdb->prefix}terms AS t
			       INNER JOIN {$wpdb->prefix}term_taxonomy AS tt
			               ON tt.parent = t.term_id
			       INNER JOIN {$wpdb->prefix}terms AS ct
			               ON ct.term_id = tt.term_taxonomy_id
			              AND ct.slug != 'fijo-expres'

			 WHERE t.slug = 'ideas-expres'
			 ORDER BY
			       ct.slug = 'ultimos-dias' DESC,  -- últimos días first position
			       ct.term_order ASC,
			       ct.term_id ASC
		";
		

		return $wpdb->get_results($query, ARRAY_A);
	}

	/**
	 * Get the data from the children of a concrete promo.
	 *
	 * @param string $parent_slug
	 * @return array with ['id', 'slug', 'name'].
	 */
	private function getChildrenPromoData( $parent_slug ) {
		global $wpdb;

		$query = "
			SELECT ct.term_id AS id,
			       ct.slug    AS slug,
			       ct.name    AS name
			  FROM {$wpdb->prefix}terms AS t
			       INNER JOIN {$wpdb->prefix}term_taxonomy AS tt
			               ON tt.parent = t.term_id
			       INNER JOIN {$wpdb->prefix}terms AS ct
			               ON ct.term_id = tt.term_taxonomy_id
			              AND ct.slug != 'fijo-expres'

			 WHERE t.slug = '{$parent_slug}'
			 ORDER BY
			       ct.slug = 'ultimos-dias' DESC,  -- últimos días first position
			       ct.term_order ASC,
			       ct.term_id ASC
		";
		

		return $wpdb->get_results($query, ARRAY_A);
	}


	/**
	 * Get the expres promos' data.
	 *
	 * @return array with ['id', 'slug', 'name'].
	 */
	private function getSuperLikePromoData() {
		global $wpdb;

		$query = "
			SELECT term_id AS id,
			       slug    AS slug,
			       name    AS name
			  FROM {$wpdb->prefix}terms
			 WHERE slug = 'superlikes'
		";

		return $wpdb->get_results($query, ARRAY_A);
	}


	/*
	 * Check if a product is superlike or not
	 * 
	 * @param int $productId
	 * 
	 * @return boolean
	 */
	public function isSuperlike($productId) {
		if (!$this->belongsToPromo($productId, ['superlikes'])) {
			return false;
		}

		$lifeStart  = $this->getDateRangeBySlug('superlikes')['life_start'];
		$promoStart = $this->getPromoData($productId)['start_date'];
		$promoStart = strtotime($promoStart);

		return ($lifeStart == $promoStart);
	}


	/*
	 * Check if a product is express or not and returns which day it belongs to
	 *
	 * @param int $productId
	 *
	 * @return string|null
	 */
	public function isExpress( $productId ) {
		
		$startDate = get_post_meta( $productId, 'wcaldi_promo_startdate', TRUE );

		$express = null;

		if( $this->belongsToPromo( $productId, ['miercoles-expres'] ) ) {
			$promoDates = $this->getDateRangeBySlug( 'miercoles-expres' );
			$promoStart = date('Y-m-d',$promoDates['life_start']);
			if( $promoStart == $startDate ||
				$promoStart.' 00:00:00' == $startDate ) {
					$express = 'miercoles-expres';
			}
		}
		if( $this->belongsToPromo( $productId, ['jueves-expres'] ) ) {
			$promoDates = $this->getDateRangeBySlug( 'jueves-expres' );
			$promoStart = date('Y-m-d',$promoDates['life_start']);
			if( $promoStart == $startDate ||
				$promoStart.' 00:00:00' == $startDate ) {
					$express = 'jueves-expres';
			}
		}
		if( $this->belongsToPromo( $productId, ['sabado-expres'] ) ) {
			$promoDates = $this->getDateRangeBySlug( 'sabado-expres' );
			$promoStart = date('Y-m-d',$promoDates['life_start']);
			if( $promoStart == $startDate ||
				$promoStart.' 00:00:00' == $startDate ) {
					$express = 'sabado-expres';
			}
		}
		if( $this->belongsToPromo( $productId, ['ideas-expres-lunes'] ) ) {
			$promoDates = $this->getDateRangeBySlug( 'lunes-expres' );
			$promoStart = date('Y-m-d',$promoDates['life_start']);
			if( $promoStart == $startDate ||
				$promoStart.' 00:00:00' == $startDate ) {
					$express = 'lunes-expres';
			}
		}
		if( $this->belongsToPromo( $productId, ['ideas-expres-miercoles'] ) ) {
			$promoDates = $this->getDateRangeBySlug( 'ideas-expres-miercoles' );
			$promoStart = date('Y-m-d',$promoDates['life_start']);
			if( $promoStart == $startDate ||
				$promoStart.' 00:00:00' == $startDate ) {
					$express = 'miercoles';
			}
		}
		if( $this->belongsToPromo( $productId, ['entre-plantas'] ) ) {
			$promoDates = $this->getDateRangeBySlug( 'entre-plantas' );
			$promoStart = date('Y-m-d',$promoDates['life_start']);
			if( $promoStart == $startDate ||
				$promoStart.' 00:00:00' == $startDate ) {
					$express = 'jueves-expres';
			}
		}
		if( $this->belongsToPromo( $productId, ['ideas-expres-sabado'] ) ) {
			$promoDates = $this->getDateRangeBySlug( 'ideas-expres-sabado' );
			$promoStart = date('Y-m-d',$promoDates['life_start']);
			if( $promoStart == $startDate ||
				$promoStart.' 00:00:00' == $startDate ) {
					$express = 'sabado-expres';
			}
		}

		return $express;

	}

	/**
	 * Check if a product belongs to an array of promos
	 * 
	 * @param int $productId
	 * @param array $promoSlugs - Array of slugs of promos - ['promo1','promo2']
	 * 
	 * @return boolean
	 */
	public function belongsToPromo( $productId, $promoSlugs ) {

		$promos = $this->getProductPromos( $productId );
		$belongs = false;

		foreach( $promos as $p ) {

			foreach( $promoSlugs as $s ) {

				if( $p['slug'] == $s ) {
					$belongs = true;
					break;
				}

			}

		}

		return $belongs;

	}

	/**
	 * Get visibility range of promos.
	 * 
	 * @param string $slug - The express promo slug to retrieve range from
	 * @param int $addWeek - It increases or reduces the current week in a number. Optional - Default null
	 *
	 * @return array with ['start_date', 'end_date'].
	 */
	public function getDateRangeBySlug($slug, $addWeek = 0) {
		$currentDayNum = date('w');
		$addSeconds    = $addWeek * 7 * 24 * 3600;

		$lifeStartTime = strtotime('monday this week');
		$lifeEndTime   = strtotime('sunday this week 23:59:59');
		$viewStartTime = strtotime('monday this week');
		$viewEndTime   = strtotime('sunday this week 23:59:59');

		switch ($slug) {

			// OLD Promotions - Delete after the final implementation of the new promotions is working correctly
			case 'miercoles-expres':
				if ($currentDayNum === '6' || $currentDayNum === '0') {
					// Sábado y domingo.
					$lifeStartTime = strtotime('wednesday next week');
					$lifeEndTime   = strtotime('friday next week 23:59:59');
				} else {
					// Lunes, martes, miércoles, jueves y viernes.
					$lifeStartTime = strtotime('wednesday this week');
					$lifeEndTime   = strtotime('friday this week 23:59:59');
				}

				$viewStartTime = strtotime('saturday last week');
				$viewEndTime   = strtotime('friday this week 23:59:59');

				break;

			case 'jueves-expres':
				if ($currentDayNum === '0') {
					// Domingo.
					$lifeStartTime = strtotime('thursday next week');
					$lifeEndTime   = strtotime('saturday next week 23:59:59');
				} else {
					// Lunes, martes, miércoles, jueves, viernes y sábado.
					$lifeStartTime = strtotime('thursday this week');
					$lifeEndTime   = strtotime('saturday this week 23:59:59');
				}

				$viewStartTime = strtotime('sunday last week');
				$viewEndTime   = strtotime('saturday this week 23:59:59');

				break;

			case 'sabado-expres':
				if ($currentDayNum === '6' || $currentDayNum === '0') {
					// Sábado y domingo.
					$lifeStartTime = strtotime('saturday next week');
					$lifeEndTime   = strtotime('tuesday +2 weeks 23:59:59');
				} else {
					// Lunes, martes, miércoles, jueves y viernes.
					$lifeStartTime = strtotime('saturday this week');
					$lifeEndTime   = strtotime('tuesday next week 23:59:59');
				}

				$viewStartTime = strtotime('saturday last week');
				$viewEndTime   = strtotime('tuesday next week 23:59:59');

				break;

			case 'ultimos-dias':
				if ($currentDayNum === '6' || $currentDayNum === '0') {
					// Sábado y domingo.
					$lifeStartTime = strtotime('saturday this week');
					$lifeEndTime   = strtotime('tuesday next week 23:59:59');
				} elseif ($currentDayNum === '1' || $currentDayNum === '2') {
					// Lunes y martes.
					$lifeStartTime = strtotime('saturday last week');
					$lifeEndTime   = strtotime('tuesday this week 23:59:59');
				}

				$viewStartTime = strtotime('saturday last week');
				$viewEndTime   = strtotime('tuesday this week 23:59:59');

				break;

			case 'superlikes':
					$lifeStartTime = strtotime('monday this week');
					$lifeEndTime   = strtotime('sunday this week 23:59:59');

					$viewStartTime = strtotime('monday this week');
					$viewEndTime   = strtotime('sunday this week 23:59:59');
				break;

			// NEW Promotions
			case 'ideas-expres-lunes':

				$lifeStartTime = strtotime('monday this week');
				$lifeEndTime   = strtotime('tuesday this week 23:59:59');

				break;

			case 'ideas-expres-miercoles':
			case 'entre-plantas':

				$lifeStartTime = strtotime('wednesday this week');
				$lifeEndTime   = strtotime('friday this week 23:59:59');

				break;

			case 'ideas-expres-sabado':

				$lifeStartTime = strtotime('saturday this week');
				$lifeEndTime   = strtotime('sunday this week 23:59:59');

				break;

			case 'ofertas-semana-frescos':
			case 'especiales-semana':

				$lifeStartTime = strtotime('monday this week');
				$lifeEndTime   = strtotime('sunday this week 23:59:59');

				break;

			case 'gran-tematica':

				$week = (int) date("W");
				if( $week % 2 == 0 ) {
					$lifeStartTime = strtotime('monday this week');
					$lifeEndTime   = strtotime('sunday next week 23:59:59');
				} else {
					$lifeStartTime = strtotime('monday last week');
					$lifeEndTime   = strtotime('sunday this week 23:59:59');
				}
				break;

			case 'destacados-fin-semana-frescos':
			case 'destacados-fin-semana-general':

				$lifeStartTime = strtotime('friday this week');
				$lifeEndTime   = strtotime('sunday this week 23:59:59');

				break;
		}

		return [
			'life_start' => $lifeStartTime + $addSeconds,
			'life_end'   => $lifeEndTime + $addSeconds,
			'view_start' => $viewStartTime + $addSeconds,
			'view_end'   => $viewEndTime + $addSeconds
		];
	}


	/**
	 * It allows to change the current week increasing or reducing weeks
	 * 
	 * @param int $year
	 * @param int $week
	 * @param int $increase - Optional
	 * 
	 * @return array - ['week' => , 'year' => ]
	 */
	public function calculateWeek( $year, $week, $increase = null ) {

		$requestedYear = $year;
		$requestedWeek = $week;

		if( $increase != null ) {
			$requestedWeek = $requestedWeek + $increase;
			if( $requestedWeek < 1 ) {
				$requestedWeek = 52 + $requestedWeek;
				$requestedYear--;
			} else if ( $requestedWeek > 52 ) {
				$requestedWeek = $requestedWeek - 52;
				$requestedYear++;
			}
		}

		$requestedWeek = str_pad( $requestedWeek, 2, "0", STR_PAD_LEFT);

		return [
			'week' => $requestedWeek,
			'year' => $requestedYear
		];

	}


	/**
	 * It returns the promos the product belongs to
	 * 
	 * @param int $productId
	 * @param boolean $onlyCurrent - It only returns the promos that are currently running - Default true
	 * 
	 * @return array
	 */
	public function getProductPromos( $productId, $onlyCurrent = true ) {

		$productPromos = [];

		if( $onlyCurrent ) {
			$startDate = strtotime( $this->getPromoData( $productId )['start_date'] );
		}

		if ( $terms = wp_get_post_terms( $productId, $this->taxonomy ) ) {

			$promosRepo = new PromosRepo();

            foreach ( $terms as $t ) {

				$promoDates = $this->getDateRangeBySlug( $t->slug );
				if( $promoDates['life_start'] == $startDate ||
					$promoDates['life_start'] == $startDate.' 00:00:00' ||
					!$onlyCurrent ) {
						$additionalData = $promosRepo->getAdditionalPromoData( $t->term_id );
						
						if( $additionalData['active'] ) {
							$productPromos[] = [
								'id'                 => $t->term_id,
								'name'               => $t->name,
								'slug'               => $t->slug,
								'alternative_title'  => $additionalData['alternative_title'],
								'icon'               => $additionalData['icon'],
								'header_img'         => $additionalData['header_img'],
								'price_img'          => $additionalData['price_img'],
								'topo'               => $additionalData['topo'],
								'color'              => $additionalData['color'],
								'color_text'         => $additionalData['color_text'],
								'general_list'       => $additionalData['general_list'],
								'alternative_prices' => $additionalData['alternative_prices'],
								'show_subcategories' => $additionalData['show_subcategories'],
								'legal'              => $additionalData['legal'],
								'active'             => $additionalData['active'],
								'level'              => $additionalData['level']
							];
						}

				}

				

            }
            
		}

		return $productPromos;

	}

	/**
	 * It returns the legal texts of the promos the product belongs to
	 * 
	 * @param int $productId
	 * @param boolean $onlyCurrent - It only returns the promos that are currently running - Default true
	 * 
	 * @return array
	 */
	public function getLegalPromos( $productId ) {

		$legals = [];

		if ( $terms = wp_get_post_terms( $productId, $this->taxonomy ) ) {

			$promosRepo = new PromosRepo();

            foreach ( $terms as $t ) {

				$additionalData = $promosRepo->getAdditionalPromoData( $t->term_id );
				
				if( $additionalData['active'] && $additionalData['legal'] != null ) {
					$legals[] = $additionalData['legal'];
				}
			}

		}

		return $legals;
	}

	/**
	 * It returns the promo prices
	 * 
	 * @param int $productId
	 * 
	 * @return array - [ 'normal_price' => , 'offer_price' => , 'normal_unit_price' => , 'offer_unit_price' => ]
	 */
	public function getPromoData( $productId ) {

		return $prices = [
			'start_date'			=> get_post_meta( $productId, 'wcaldi_promo_startdate', TRUE ),
			'end_date'				=> get_post_meta( $productId, 'wcaldi_promo_enddate', TRUE ),
			'normal_price' 			=> get_post_meta( $productId, 'wcaldi_promo_normal_price', TRUE ),
			'offer_price' 			=> get_post_meta( $productId, 'wcaldi_promo_offer_price', TRUE ),
			'normal_unit_price' 	=> get_post_meta( $productId, 'wcaldi_promo_normal_unit_price', TRUE ),
			'offer_unit_price' 		=> get_post_meta( $productId, 'wcaldi_promo_offer_unit_price', TRUE ),
			'discount' 				=> get_post_meta( $productId, 'wcaldi_promo_discount', TRUE ),
			'palet'			 		=> get_post_meta( $productId, 'wcaldi_promo_palet', TRUE )
		];

	}

	/**
	 * It returns the parent promo (the top one)
	 * 
	 * @param string $promo_slug
	 * 
	 * @return array
	 */
	public function getParentPromo( $promo_id ) {

		return get_term( end( get_ancestors($promo_id, WC_ALDI_TAX_PROMOS) ), WC_ALDI_TAX_PROMOS );

	}

	/**
	 * It returns an array with the data of the subcategory (Level 2). The subcategory is the one with the settings of the promo
	 * 
	 * @param array $product_promos - An array with the data of all the promotions
	 */
	public function getCategoryStyle( $product_promos ) {

		$subcategory = null;

		if( count( $product_promos ) == 1 ) {
			return $product_promos[0];
		}

		foreach( $product_promos as $p ) {

			// Giving priority to chep-palet
			if( $p['slug'] == 'chep-palet' ) {
				$subcategory = $p;
				break;
			}

			// Getting the second level
			if( $p['level'] == 2 ) {
				$subcategory = $p;
			}
		}

		return $subcategory;

	}

}
