<?php

namespace WCAldi\Repositories;

use WCAldi\Repositories\TaxonomyRepo;
use WCAldi\Importer\ProductImporter;

class PromosRepo {

    protected $promos;

    /**
     * It returns all the existing promos
     *
	 * @return array
     */
    public function getAll() {

        $terms = get_terms( array(
            'taxonomy' => WC_ALDI_TAX_PROMOS,
            'hide_empty' => false,
        ));

        return $terms;

    }

    /**
     * It returns the promos of a single product
     *
     * @param int $product_id
     *
	 * @return array of the terms
     */
    public function getProductPromos( $product_id ) {

        $terms = wp_get_post_terms( $product_id, WC_ALDI_TAX_PROMOS, array() );

        return $terms;

    }

    /**
     * It deletes the current existing promos for a product, and adds the new one
     *
     * @param int $product_id
     * @param array $promos - An array of names of the promos. Use '|' for hierarchy. For example 'Promo 1|Subpromo 1-1|Subsubpromo 1-1-1'
     */
    public function setProductPromos( $product_id, $promos ) {

        $product_tax = new TaxonomyRepo();
        $productImporter = new ProductImporter();
        $promosTax[]['name'] = $productImporter->transformPromoToWordpress( $promos['full_route'] );
        $product_tax->setProductTaxonomies( $product_id, $promosTax, WC_ALDI_TAX_PROMOS ); // We use this function for selecting the promos the product belongs to

        $this->setProductPromoData( $product_id, $promos ); // Saving the data of the promo

    }

    /**
     * It saves the data for a product
     *
     * @param $product_id - The id of the product
     * @param array $promoData - An array with all the promo data 
     */
    public function setProductPromoData( $product_id, $promoData ) {

        $promoData = (array) $promoData;

        if( !empty($promoData) ) {
            update_post_meta( $product_id, 'wcaldi_promo_startdate', $promoData['start_date'] );
            update_post_meta( $product_id, 'wcaldi_promo_enddate', $promoData['end_date'] );
            update_post_meta( $product_id, 'wcaldi_promo_normal_price', $promoData['normal_price'] );
            update_post_meta( $product_id, 'wcaldi_promo_offer_price', $promoData['offer_normal_price'] );
            update_post_meta( $product_id, 'wcaldi_promo_normal_unit_price', $promoData['unit_price'] );
            update_post_meta( $product_id, 'wcaldi_promo_offer_unit_price', $promoData['offer_unit_price'] );
        } else {
            // Deleting data from promo
            update_post_meta( $product_id, 'wcaldi_promo_startdate', '' );
            update_post_meta( $product_id, 'wcaldi_promo_enddate', '' );
            update_post_meta( $product_id, 'wcaldi_promo_normal_price', '' );
            update_post_meta( $product_id, 'wcaldi_promo_offer_price', '' );
            update_post_meta( $product_id, 'wcaldi_promo_normal_unit_price', '' );
            update_post_meta( $product_id, 'wcaldi_promo_offer_unit_price', '' );
        }

        if( $promoData['discount'] != '' ) {
            update_post_meta( $product_id, 'wcaldi_promo_discount', $promoData['discount'] );
        } else {
            update_post_meta( $product_id, 'wcaldi_promo_discount', '' );
        }

        if( $promoData['palet'] != '' ) {
            update_post_meta( $product_id, 'wcaldi_promo_palet', $promoData['palet'] );
        } else {
            update_post_meta( $product_id, 'wcaldi_promo_palet', '' );
        }

    }

    /**
     * It gets the data stored as metas: alternative_title, icon, header_img, price_img, color, general_list, alternative_prices, show_subcategories, active
     * 
     * @param int $promo_id
     * 
     * @return array 
     */
    public function getAdditionalPromoData( $promo_id ) {

        $promo_data = [];

        $general_list       = get_field('promo_exclude', 'product_cat_'.$promo_id );
        $alternative_prices = get_field('promo_change_price', 'product_cat_'.$promo_id );
        $show_subcategories = get_field('promo_show_subcategories', 'product_cat_'.$promo_id );
        $active             = get_field('promo_is_active', 'product_cat_'.$promo_id );

        $promo_data = [
            'alternative_title'  => get_field('promo_title', 'product_cat_'.$promo_id ),
            'icon'               => get_field('promo_icon', 'product_cat_'.$promo_id ),
            'header_img'         => get_field('promo_header_img', 'product_cat_'.$promo_id ),
            'price_img'          => get_field('promo_price_img', 'product_cat_'.$promo_id ),
            'topo'               => get_field('promo_topo', 'product_cat_'.$promo_id ),
            'color'              => get_field('promo_color', 'product_cat_'.$promo_id ),
            'color_text'         => get_field('promo_color_text', 'product_cat_'.$promo_id ),
            'general_list'       => ( $general_list === null ) ? false : $general_list,
            'alternative_prices' => ( $alternative_prices === null ) ? true : $alternative_prices,
            'show_subcategories' => ( $show_subcategories === null ) ? true : $show_subcategories,
            'legal'              => get_field('promo_legal', 'product_cat_'.$promo_id ),
            'active'             => ( $active === null ) ? true : $active,
            'level'              => $this->getPromoLevel( $promo_id )
        ];

        return $promo_data;

    }

    /**
    * It indicates which level the promo belongs to. 1, 2, 3
    *
    * @param int $promoId
    * @param int $level - Default 1
    *
    * @return int 1,2,3 
    */
    public function getPromoLevel( $promoId, $level = 1 ) {
        
        $term = get_term( $promoId );

        if( $term->parent != 0 ) {

            $level = $this->getPromoLevel( $term->parent, $level +1 );

        }

        return $level;

    }

}